@@ -55,6 +55,17 @@ class TestCookieRedirectingFormPlugin(FixtureBase):
             logout_handler_path, rememberer_name, **kw)
         return plugin
 
+    def test_make_redirecting_plugin(self):
+        from repoze.who.plugins.formcookie import make_redirecting_plugin
+        plugin = make_redirecting_plugin(login_form_path='/form',
+                                         login_handler_path='/login',
+                                         logout_handler_path='/logout',
+                                         rememberer_name='cookie')
+        self.assertEqual(plugin.login_form_path, '/form')
+        self.assertEqual(plugin.login_handler_path, '/login')
+        self.assertEqual(plugin.logout_handler_path, '/logout')
+        self.assertEqual(plugin.rememberer_name, 'cookie')
+
     def test_implements(self):
         from zope.interface.verify import verifyClass
         from repoze.who.interfaces import IIdentifier, IChallenger
@@ -123,7 +134,7 @@ class TestCookieRedirectingFormPlugin(FixtureBase):
         self.assertEqual(environ['repoze.who.application'].code, 401)
         self.assertEqual(environ['came_from'], 'http://localhost/')
 
-    def test_challange_current_url(self):
+    def test_challenge_current_url(self):
         environ = self.app.post('/test').req.environ
         result = self.plugin.challenge(environ, '401 Unauthorized', [('app', 1)], [('forget', 1)])
         self.assertEqual(result.code, 302)
@@ -131,13 +142,19 @@ class TestCookieRedirectingFormPlugin(FixtureBase):
         self.assertEqual(headers['Location'], '/loginform')
         self.failUnless('http://localhost/test' in headers['Set-Cookie'])
 
-    def test_challange_environ(self):
-        environ = self.app.post('/test', extra_environ={'came_from':'http://www.google.com'}).req.environ
+    def test_challenge_cookie(self):
+        environ = self.app.post('/test', headers={'Cookie':'came_from=http://www.google.com'}).req.environ
         result = self.plugin.challenge(environ, '401 Unauthorized', [('app', 1)], [('forget', 1)])
         self.assertEqual(result.code, 302)
         headers = dict(result.headers)
         self.assertEqual(headers['Location'], '/loginform')
-        self.failUnless('http://www.google.com' in headers['Set-Cookie'])
+
+    def test_challenge_environ(self):
+        environ = self.app.post('/test', extra_environ={'came_from':'http://www.google.com'}).req.environ
+        result = self.plugin.challenge(environ, '401 Unauthorized', [('app', 1)], [('forget', 1)])
+        self.assertEqual(result.code, 302)
+        headers = dict(result.headers)
+        self.assertEqual(headers['Location'], 'http://www.google.com')
 
     def test_remember(self):
         identity = {}
@@ -153,5 +170,42 @@ class TestCookieRedirectingFormPlugin(FixtureBase):
         result = self.plugin.forget(environ, identity)
         self.assertEqual(result, None)
         self.assertEqual(environ['repoze.who.plugins']['cookie'].forgotten,
-            identity
-        )
+            identity)
+
+    def test_force_https_pass(self):
+        self.plugin = self._make_one(force_https=True)
+        environ = self.app.post('/login',
+            params={'login':'chris', 'password':'password'}).req.environ
+        result = self.plugin.identify(environ)
+        self.failIf(result)
+        self.failUnless('force_https is set' in environ['repoze.who.logger'].messages[1])
+
+    def test_force_https_sucess(self):
+        self.plugin = self._make_one(force_https=True)
+        environ = self.app.post('/login',
+            params={'login':'chris', 'password':'password'},
+            extra_environ={'wsgi.url_scheme':'https'}).req.environ
+        result = self.plugin.identify(environ)
+        self.failUnless(result)
+
+    def test_custom_login(self):
+        self.plugin = self._make_one(login_redirect_method='/home/%(login)s/')
+        environ = self.app.post('/login',
+            params={'login':'chris', 'password':'password'}).req.environ
+        result = self.plugin.identify(environ)
+        self.failUnless(result)
+        app = environ['repoze.who.application']
+        self.assertEqual(len(app.headers), 1)
+        name, value = app.headers[0]
+        self.assertEqual(name, 'location')
+        self.assertEqual(value, '/home/chris/')
+        self.assertEqual(app.code, 302)
+
+    def test_custom_logout(self):
+        self.plugin = self._make_one(logout_redirect_method='/out/')
+        environ = self.app.post('/logout',
+            params={'login':'chris', 'password':'password'}).req.environ
+        result = self.plugin.identify(environ)
+        self.failUnless('performing logout' in environ['repoze.who.logger'].messages)
+        self.assertEqual(environ['repoze.who.application'].code, 401)
+        self.assertEqual(environ['came_from'], '/out/')
